#!/usr/bin/perl -T
#$Id: PID_fast_control.pl,v 1.14 2015/06/18 14:41:14 sqko Exp $
use strict;
use warnings;
use RFC::Rig;
use RFC::Visitor;
my $v = RFC::DisableProglogVisitor->new();
use Fcntl qw(:DEFAULT :flock);
die "No rig specified!\n" unless (@ARGV);
my $r = shift;
sleep(20) unless (@ARGV);
&main($r);
sleep(1);
exit;

sub main
{
    my $rigname = shift;
    my $rig = RFC::Rig->new($rigname);
    $rig->check_run;
    $rig->Accept($v);
    my $name = 'PID_fast_control';
    my $lock = '/home/rig'.$rig->name().'/'.$name.'.lock';
    sysopen(MFH, $lock, O_WRONLY | O_CREAT) or CORE::die "Error: Could not open $lock $!";
    my $res = flock(MFH,LOCK_EX | LOCK_NB);
    unless ($res)
    {
	$rig->DESTROY();
	return;
    }
    my $modtime = time + 600;
    my @devices = $rig->list_devices('PID');
    while (1)
    {	
	last unless (@devices);
	if (time > $modtime)
	{
	    $rig = undef;
	    select(undef,undef,undef,0.5);
	    $rig = RFC::Rig->new($rigname);
	    $rig->Accept($v);
	    $modtime = time + 600;
	    @devices = $rig->list_devices('PID');
	}
	foreach my $name (@devices)
	{
	    my $d = $rig->get_device('PID',$name);
	    next unless ($d);
	    unless ($d->fast)
	    {
		$d->remove();
		$name = '';
		next;
	    }
	    $d->init(); ## load any changes in configuration ###
	    $d->control(); ## Do calculation and control ##
	    my $res = $d->store(); ## Save state for future calls ##
	    next if ($res);
	    $rig->errorlog("Could not store PID settings, reloading.");
	    $modtime = 0; ## Force reload of rig if PID has errors ###
	    last;
	}
	sleep(1);
    }
    $rig->DESTROY();
    return;
}

