#!/usr/bin/perl -T
# $Id: rig_ajax_run.cgi,v 1.58 2015/07/02 07:59:15 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 20011             ###
###                                                      ###
############################################################
############################################################

use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;

my $acmode = get_cv('global','access_mode');
$acmode = 'celltest' unless ($acmode);

my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);
if (param('ajax'))
{
    print header();
    my $ac = param('action');
    unless ($ac)
    {
        print "";
        exit;
    }
    if ($ac eq 'get_imp_data')
    {
	my $test = param('testno');
	my $fn = param('file');
	unless ($test && $fn)
	{
	    print "Warning, No test or filename specified!\n";
	    exit;
	}
	my $impedancedir =  $RFC::Header::publicdir . "rig" . $rig->name . "/" . $rig->name . "test"  . $test . "/impedance";
	unless (-d $impedancedir)
	{
	    print "Warning, $impedancedir not found!\n";
	    exit;
	}
        my $file = $impedancedir.'/'.$fn;
        my $name = '';
        my @data = ();
        my $f = SemaforeFile->new($file);
        my @lines = $f->readlines();
        my $x = 0;
        while ($x++ < @lines)
        {
            next unless ($lines[$x] =~ /^[\s\t]*\d+[\s\t]*$/);
            last;
        }
        while ($x++ < @lines)
        {
            push @data,$lines[$x] if ($lines[$x] && $lines[$x] =~ /\d/);
        }
        if ($file =~ /([\w\d\-\.]+)$/)
        {
            $name = $1;
        }
	print "OK\n";
	print $name,"\n";
	print $file,"\n";
	print join("\n",@data);
	exit;
    }
    if ($ac eq 'show_log')
    {
        my $u = param('user');
        my $content = $rig->get_sessionlog;
        print $content;
        exit;
    }
    if ($ac eq 'show_info')
    {
        my $u = param('user');
        my $content = $rig->info();
        print $content;
        exit;
    }
    if ($ac eq 'show_error')
    {
        my $u = param('user');
        my $content = $rig->get_errorlog();
        print $content;
        exit;
    }
    if ($ac eq 'is_iv')
    {
	if ($rig->is_iv)
	{
	    print "1\n";
	}
	else
	{
	    print "0\n";
	}
        exit;
    }
    if ($ac eq 'get_callibration_file')
    {
	if (param('filename') =~ /([\w\.\-]+)/)
	{
	    my $filename = $homedir.'convert-tables/userfiles/'.$1;
	    my $file = SemaforeFile->new($filename);
	    print join("\n",$file->readlines) if ($file->exist);
	}
	exit;
    }
    if ($ac eq 'plot_callibration_file')
    {
	my $GNUPLOT = &get_cv('GNUPLOT','location');
	if (param('filename') =~ /([\w\.\-]+)/)
	{
	    my $fn = $1;
	    my $dat = $homedir.'convert-tables/userfiles/'.$fn;
	    unless (-e $dat && -s $dat > 10)
	    {
		print "";
		exit;
	    }
	    chdir $htmldir;
	    my $pwd = `pwd`;
	    my $out = 'png/'.$fn.'.png';
	    my $gp = SemaforeFile->new('/tmp/plot_cal_'.$$.'.gp');
	    my @lines = ("set out '$out'",'set term png',"set ylabel 'T (C)'","set xlabel 'V (mV)'","set size 0.7,0.7");
	    push @lines,"set title '$fn'";
	    push @lines,"set nokey";
	    push @lines,"plot '</bin/sort -n $dat' u 1:2 w linesp"; 
	    $gp->writeline(@lines);
	    my $cmd = $GNUPLOT.' '.$gp->filename;
	    system($cmd) and print "";
	    unlink $gp->filename if ($gp->exist);
	    print 'celltest/png/'.$fn.'.png'."\n";
	}
	exit;
    }
    unless (login_ok())
    {
        print "";
        exit;
    }
    my $usermode = user_auth($acmode,'c');
    if (RFC::RFCCGI::rig_mode())
    {
        ####  Read and use mode deprecated after initialisation of rig authentification ####
        $usermode = 1;
    }
    unless (RFC::RFCCGI::rig_auth())
    {
        $usermode = 0;
    }
    unless ($usermode)
    {
        print "Warning: You are not authorised to change parameters on $rig changes aborted!";
        exit;
    }
    if ($ac eq 'run_device_function')
    {
	## Although this function is mererely for getting data, it can possibly be misused ##
	## To minimise damage, it can only be called by logged in users (and if they can   ##
	## hack the login, they can do damage anyways, so no reason to be paranoid here..) ## 
	my $type = param('type');
	my $name = param('dname');
	my $function = param('function');
	my $dev = $rig->init_device($type,$name);
	if ($dev)
	{
	    eval {
		my @res = ();
		my $args = param('arguments');
		if ($args)
		{
		    @res = $dev->$function(split(/\s*\,\s*/,$args));
		}
		else
		{
		    @res = $dev->$function();
		}
		foreach (@res)
		{
		    $_ = '' unless (defined($_));
		}
		if (@res)
		{
		    print "OK\n";
		    print join("\n",@res) if (@res);
		}
		else
		{
		    print "ERROR\n";
		    print "Warning: Device function '$function' returned undefined!";
		}
	    };
	    if ($@)
	    {
		print "ERROR\n";
		if ($@ =~ /^(.*)\s*\#+/)
		{
		    print $1;
		}
		else
		{
		    print $@;
		}
	    }
	}
	else
	{
	    print "ERROR\n";
	    print "Warning: no device found of type '$type' with name '$name'!";
	}
	$rig->DESTROY();
        exit;
    }
    if ($ac eq 'list_caching')
    {
	my @list = $rig->list_caching();
	foreach my $item (@list)
	{
	    print $item->type(),' - ',$item->name(),"\n";
	}
	exit;
    }
    if ($ac eq 'get_programs_test')
    {
	my $test = $rig->session;
	$test = &get_CGI_value_clean('session') if (&get_CGI_value('session'));
	my $dir = $RFC::Header::publicdir.'rig'.$rig->name.'/'.$rig->name.'test'.$test;
	opendir(DIR,$dir);
	my @list = grep {/program_/} readdir(DIR);
	closedir DIR;
	foreach (sort {$a cmp $b} @list)
	{
	    print $dir,'/',$_,"\n";
	}
	exit;
    }
    if ($ac eq 'start_iv')
    {
        my $user = &get_user();
        my $res = CGI_client($rig->name,'iv');
        if ($res =~ /i-V curve started/)
        {
            $rig->proglog("i-V curve started",$user);
            print "i-V curve on $rig started";
        }
        else
        {
            $rig->errorlog("Error: i-V curve could not be started: $res",$user);
            print $res;
        }
        exit;
    }
    if ($ac eq 'stop_iv')
    {
        my $user = &get_user();
        my $res = CGI_client($rig->name,'stop_iv');
        if ($res =~ /stopping/i)
        {
            $rig->proglog("i-V curve stopped!",$user);
            print "i-V curve on $rig stopped";
        }
        else
        {
            $rig->errorlog("Error: i-V curve could not be stopped: $res",$user);
            print $res;
        }
        exit;
    }
    if ($ac eq 'set_imp')
    {
	my $ip = get_CGI_value_clean('server');
	my $port = get_CGI_value_clean('port');
	my $user = get_CGI_value_clean('impuser');
	my $session = get_CGI_value_clean('session');
	my $file = get_CGI_value_clean('comp');
	my $comploc = get_cv('impedance','standard_compensation_files');
	$rig->change_config_value('impedance','server',$ip);
	$rig->change_config_value('impedance','port',$port);
	$rig->change_config_value('impedance','user',$user);
	$rig->change_config_value('impedance','session',$session);
	$rig->change_config_value('impedance','compensation',$file);
	my $dir;
	if ($comploc)
	{
	    $comploc =~ /^([\w\/]+)\*/;
	    $dir = $1;
	}
	else
	{
	    $dir = $RFC::Header::publicdir.'rig'.$rig->name.'/imp_comp/';
	}
	my $impcmd = 'impedance';
	if ($file eq 'DEFAULT')
	{
	    $dir = '';
	    $file = '';
	    $impcmd = 'current_impedance';
	}
	my $string = "$impcmd $ip:$port $session $user comp:$dir$file"; 
	#### Check if program ia already running and abort if so ###
	my $isused = 0;
	my $cmd = '/bin/ps -efl | /bin/grep custom_prog.pl | /bin/grep rig'.$rig->name;
	my $str = `$cmd`;
	my @list = ();
	@list = split(/\n/,$str) if ($str);
	foreach (@list)
	{
	    $isused = 1 if /celltest/;
	}
	if ($isused)
	{
	    print "Program already running, no new program can be activated, aborting!";
	    exit;
	}
        ##### Create new unique file name for program name ####
        my $dat = `date +"%Y:%m:%d:%H:%M"`;
        $dat =~ /([\d\:]*)/;
        $dat = $1;
        my $file1 = $RFC::Header::publicdir.'rig'.$rig->name.'/'.$rig->name.'test'.$rig->session.'/impedance_'.$dat.'.txt';
	my $f = SemaforeFile->new($file1);
	$f->append($string);
	my $res = RFC::Main::CGI_client($rig->name,'custom',$f->filename,get_user());
	chomp $res;
	print $res;
	exit;
    }
    if ($ac eq 'set_multiplex')
    {
        my $dev = get_CGI_value_clean('dev');
        my $gas = get_CGI_value_clean('gas');
        my $user = &get_user();
        my $d = $rig->get_device('multiplex',$dev);
        if ($d) {
            eval {
		$d->set($gas,$user);
                if ($d->mode =~ /auto/i)
		{
		    print "Gas for multiplexer $dev set to $gas for $rig";
		    $rig->proglog("Gas for multiplexer $dev set to $gas",$user);
		}
		else
		{
		    print "Gas for multiplexer $dev set to $gas manually for $rig";
		    $rig->proglog("Gas for multiplexer $dev set to $gas manually",$user);
		}
            };
            if ($@)
            {
                if ($@ =~ /(.*)\#+/)
                {
                    my $s = $1;
                    $rig->errorlog("Error when user $user attempts to set gas for multiplexer $dev: $s",$user);
                    print "Error: ".$s;
                }
                else
                {
                    $rig->errorlog("Error when user $user attempts to set gas for multiplexer $dev: $@",$user);
                    print "Error: ".$@;
                }
            }
        }
        else
        {
            print "Warning: No gas multiplexer device found with name '$dev' for $rig";
        }
        exit;

    }
    if ($ac eq 'set_relay')
    {
        my $dev = get_CGI_value_clean('dev');
        my $status = get_CGI_value_clean('status');
	$status = 0 unless ($status);
        my $user = &get_user();
        my $d = $rig->get_device('relay',$dev);
        if ($d) {
            eval {
                if ($d->mode =~ /auto/i)
		{
		    $d->set($status,$user);
		    print "Status for relay $dev set to $status for $rig";
		}
		else
		{
		    my $f = $d->set($status,$user);
		    die "Status for $dev could not be set to $status for $rig\n" unless ($f == $status);
		    print "Status for $dev set to $status manually for $rig";
		}
            };
            if ($@)
            {
                if ($@ =~ /(.*)\#+/)
                {
                    my $s = $1;
                    $rig->errorlog("Error when user $user attempts to set status for device $dev: $s",$user);
                    print "Error: ".$s;
                }
                else
                {
                    $rig->errorlog("Error when user $user attempts to set status for device $dev: $@",$user);
                    print "Error: ".$@;
                }
            }
        }
        else
        {
            print "Warning: No relay device found with name '$dev' for $rig";
        }
        exit;
    }
    if ($ac eq 'set_gas')
    {
        my $dev = get_CGI_value_clean('dev');
        my $flow = get_CGI_value_clean('flow');
        my $user = &get_user();
        my $d = $rig->get_device('gas',$dev);
        if ($d) {
            eval {
                if ($d->mode =~ /auto/i)
                {
                    my $cutoff = $d->cutoff_set();
                    if (!$d->is_pressure && $cutoff && $flow < $cutoff)
                    {
			my $f = $d->setflow(0,$user);
			if ($f && $f =~ /^(.+):/)
			{
			    $f = $1;
			}
			die "Flow for gas $dev could not be set to $flow l/hour for $rig\n" unless (abs($f) < $d->accuracy);
			my $str = "Flow for gas $dev set to 0 l/hour as flow was below cutoff for gas $dev on $rig";
			print $str;
			$rig->proglog($str,$user);
                    }
                    else
                    {
			my $f = $d->setflow($flow,$user);
			if ($f && $f =~ /^(.+):/)
			{
			    $f = $1;
			}
			if ($d->is_pressure)
			{
			    die "Pressure for gas $dev could not be set to $flow barA for $rig\n" unless (abs($f-$flow) < $d->accuracy);
			    print "Pressure for gas $dev set to $flow barA for $rig";
			}
			else
			{
			    die "Flow for gas $dev could not be set to $flow l/hour for $rig\n" unless (abs($f-$flow) < $d->accuracy);
			    print "Flow for gas $dev set to $flow l/hour for $rig";
			}
                    }
                }
                else
                {
		    my $f = $d->setflow($flow,$user);
                    print "Flow for gas $dev has to be set to $flow l/hour manually for $rig";
		    ### Print any slave device info as well ###
		    if ($f =~ /^[\d\.\+\-eE]+\D+/)
		    {
			$f =~ /^[\d\.\+\-eE]+(.*)/;
			print ' '.$1;
		    }
                }
            };
            if ($@)
            {
                if ($@ =~ /(.*)\#+/)
                {
                    my $s = $1;
                    $rig->errorlog("Error when user $user attempts to set flow for gas $dev: $s",$user);
                    print "Error: ".$s;
                }
                else
                {
                    $rig->errorlog("Error when user $user attempts to set flow for gas $dev: $@",$user);
                    print "Error: ".$@;
                }
            }
        }
        else
        {
            print "Warning: No gas device found with name '$dev' for $rig";
        }
        exit;
    }
    if ($ac eq 'set_pid')
    {
        my $dev = get_CGI_value_clean('dev');
        my $flow = get_CGI_value('setpoint');
	$flow = 0 unless ($flow);
	$flow =~ /([\d\.\-\+eE]+)/;
	$flow = $1+0;
        my $user = &get_user();
        my $d = $rig->get_device('PID',$dev);
        if ($d) {
            eval {
		my $f = $d->set($flow,$user);
		die "Setpoint for PID regulator $dev could not be set to $flow for $rig\n" unless ($f == $flow);
		print "Setpoint for PID $dev set to $flow for $rig";
            };
            if ($@)
            {
                if ($@ =~ /(.*)\#+/)
                {
                    my $s = $1;
                    $rig->errorlog("Error when user $user attempts to set setpoint for PID regulator $dev: $s",$user);
                    print "Error: ".$s;
                }
                else
                {
                    $rig->errorlog("Error when user $user attempts to set setpoint for PID regulator $dev: $@",$user);
                    print "Error: ".$@;
                }
            }
        }
        else
        {
            print "Warning: No PID device found with name '$dev' for $rig";
        }
        exit;
    }
    if ($ac eq 'set_temp')
    {
        my $dev = get_CGI_value_clean('dev');
        my $temp = get_CGI_value_clean('temp');
        my $ramp = get_CGI_value_clean('ramp');
        my $user = &get_user();
        my $d = $rig->get_device('tempcontrol',$dev);
        if ($d) {
            eval {
                $d->set_temp($temp,$user);
                $d->set_ramp($ramp,$user);
                print "Temperature set to $temp C and ramprate set to $ramp C/hour for $rig";
            };
            if ($@)
            {
                $rig->errorlog("Error when user $user attempts to set temperature for device $dev: $@",$user);
                print "Error: ".$@;
            }
        }
        else
        {
            print "Warning: No temperature control device found with name '$dev' for $rig";
        }
        exit;
    }
    if ($ac eq 'set_current')
    {
        my $dev = get_CGI_value('dev');
        my $c = get_CGI_value('current');
	if ($c =~ /^[\s\t]*([\+\-\.\deE]+)[\s\t]*$/)
	{
	    $c = $1;
	}
	else
	{
	    $c = 'ocv';
	}
        my $user = &get_user();
	if ($dev =~ /(.*)\s+\(analog\)/)
	{
	    my $analog = $1;
	    my $d = $rig->get_device('analog',$analog);
	    if ($d) {
		eval {
		    $d->set($c*1000);
		    print "Current output '$analog' set to $c mA for $rig";
		    $rig->proglog("Current for device '$analog' set to $c mA",$user);
		};
		if ($@)
		{
		    $rig->errorlog("Error when attempt to set current for device $analog: $@",$user);
		    print "Error: ".$@;
		}
	    }
	    else
	    {
		print "Warning: No analog output device found with name '$analog' for $rig";
	    }
	    exit;	    
	}
	else
	{
	    my $d = $rig->get_device('PSU',$dev);
	    if ($d) {
		eval {
		    $d->current($c,$user);
		    if ($c =~ /ocv/i)
		    {
			print "Current set to OCV for $rig";
		    }
		    else
		    {
			print "Current set to $c A for $rig";
		    }
		};
		if ($@)
		{
		    $rig->errorlog("Error when attempt to set current for device $dev: $@",$user);
		    print "Error: ".$@;
		}
	    }
	    else
	    {
		print "Warning: No current control device found with name '$dev' for $rig";
	    }
	}
	exit;
    }
    if ($ac eq 'set_voltage')
    {
        my $dev = get_CGI_value('dev');
        my $c = get_CGI_value('voltage');
	if ($c =~ /^[\s\t]*([\+\-\.\deE]+)[\s\t]*$/)
	{
	    $c = $1;
	}
	else
	{
	    $c = 'ocv';
	}
        my $user = &get_user();
	if ($dev =~ /(.*)\s+\(analog\)/)
	{
	    my $analog = $1;
	    my $d = $rig->get_device('analog',$analog);
	    if ($d) {
		eval {
		    $d->set($c);
		    print "Voltage output '$analog' set to $c V for $rig";
		    $rig->proglog("Voltage for device '$analog' set to $c V",$user);
		};
		if ($@)
		{
		    $rig->errorlog("Error when attempt to set voltage for device $analog: $@",$user);
		    print "Error: ".$@;
		}
	    }
	    else
	    {
		print "Warning: No analog output device found with name '$analog' for $rig";
	    }
	    exit;	    
	}
	else
	{
	    my $d = $rig->get_device('PSU',$dev);
	    if ($d) {
		eval {
		    $d->voltage($c,$user);
		    if ($c =~ /ocv/i)
		    {
			print "Voltage set to OCV for $rig";
		    }
		    else
		    {
			print "Voltage set to $c V for $rig";
		    }
		};
		if ($@)
		{
		    $rig->errorlog("Error when attempt to set voltage for device $dev: $@",$user);
		    print "Error: ".$@;
		}
	    }
	    else
	    {
		print "Warning: No voltage control device found with name '$dev' for $rig";
	    }
	}
        exit;
    }
    if ($ac eq 'add_info')
    {
        my $string = get_CGI_value('comments');
        $string =~ s/\\n/\n/g;
        my $time = localtime();
        my $user = &get_user();
        my $str = $time.', User '.$user.' added: '.$string."\n";
        my $content = $rig->info($str);
        print $content;
        exit;
    }
}
